@@ -5,8 +5,9 @@
 // This file is licensed under the GPLv2 or later
 //
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
-// Copyright (C) 2016, 2017, 2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2017, 2020 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -51,7 +52,7 @@ void JSInfo::printJS(const GooString *js) {
   gfree(u);
 }
 
-void JSInfo::scanLinkAction(LinkAction *link, const char *action, bool deleteLink) {
+void JSInfo::scanLinkAction(LinkAction *link, const char *action) {
   if (!link)
     return;
 
@@ -59,10 +60,11 @@ void JSInfo::scanLinkAction(LinkAction *link, const char *action, bool deleteLin
     hasJS = true;
     if (print) {
       LinkJavaScript *linkjs = static_cast<LinkJavaScript *>(link);
-      const GooString *s = linkjs->getScript();
-      if (s && s->c_str()) {
+      if (linkjs->isOk()) {
+	const std::string& s = linkjs->getScript();
 	fprintf(file, "%s:\n", action);
-	printJS(s);
+	GooString gooS = GooString(s);
+	printJS(&gooS);
 	fputs("\n\n", file);
       }
     }
@@ -70,20 +72,16 @@ void JSInfo::scanLinkAction(LinkAction *link, const char *action, bool deleteLin
 
   if (link->getKind() == actionRendition) {
     LinkRendition *linkr = static_cast<LinkRendition *>(link);
-    if (linkr->getScript()) {
+    if (!linkr->getScript().empty()) {
       hasJS = true;
       if (print) {
-        const GooString *s = linkr->getScript();
-        if (s && s->c_str()) {
-          fprintf(file, "%s (Rendition):\n", action);
-          printJS(s);
-          fputs("\n\n", file);
-        }
+        fprintf(file, "%s (Rendition):\n", action);
+        const GooString s(linkr->getScript());
+        printJS(&s);
+        fputs("\n\n", file);
       }
     }
   }
-  if (deleteLink)
-    delete link;
 }
 
 void JSInfo::scanJS(int nPages) {
@@ -122,15 +120,15 @@ void JSInfo::scan(int nPages) {
   }
 
   // document actions
-  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionCloseDocument),
+  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionCloseDocument).get(),
                  "Before Close Document");
-  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionSaveDocumentStart),
+  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionSaveDocumentStart).get(),
                  "Before Save Document");
-  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionSaveDocumentFinish),
+  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionSaveDocumentFinish).get(),
                  "After Save Document");
-  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionPrintDocumentStart),
+  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionPrintDocumentStart).get(),
                  "Before Print Document");
-  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionPrintDocumentFinish),
+  scanLinkAction(doc->getCatalog()->getAdditionalAction(Catalog::actionPrintDocumentFinish).get(),
                  "After Print Document");
 
   // form field actions
@@ -141,14 +139,14 @@ void JSInfo::scan(int nPages) {
       for (int j = 0; j < field->getNumWidgets(); j++) {
 	FormWidget *widget = field->getWidget(j);
 	scanLinkAction(widget->getActivationAction(),
-                       "Field Activated", false);
-	scanLinkAction(widget->getAdditionalAction(Annot::actionFieldModified),
+                       "Field Activated");
+	scanLinkAction(widget->getAdditionalAction(Annot::actionFieldModified).get(),
                        "Field Modified");
-	scanLinkAction(widget->getAdditionalAction(Annot::actionFormatField),
+	scanLinkAction(widget->getAdditionalAction(Annot::actionFormatField).get(),
                        "Format Field");
-	scanLinkAction(widget->getAdditionalAction(Annot::actionValidateField),
+	scanLinkAction(widget->getAdditionalAction(Annot::actionValidateField).get(),
                        "Validate Field");
-	scanLinkAction(widget->getAdditionalAction(Annot::actionCalculateField),
+	scanLinkAction(widget->getAdditionalAction(Annot::actionCalculateField).get(),
                        "Calculate Field");
       }
     }
@@ -170,63 +168,63 @@ void JSInfo::scan(int nPages) {
     if (!page) continue;
 
     // page actions (open, close)
-    scanLinkAction(page->getAdditionalAction(Page::actionOpenPage), "Page Open");
-    scanLinkAction(page->getAdditionalAction(Page::actionClosePage), "Page Close");
+    scanLinkAction(page->getAdditionalAction(Page::actionOpenPage).get(), "Page Open");
+    scanLinkAction(page->getAdditionalAction(Page::actionClosePage).get(), "Page Close");
 
     // annotation actions (links, screen, widget)
     annots = page->getAnnots();
     for (int i = 0; i < annots->getNumAnnots(); ++i) {
       if (annots->getAnnot(i)->getType() == Annot::typeLink) {
 	AnnotLink *annot = static_cast<AnnotLink *>(annots->getAnnot(i));
-	scanLinkAction(annot->getAction(), "Link Annotation Activated", false);
+	scanLinkAction(annot->getAction(), "Link Annotation Activated");
       } else if (annots->getAnnot(i)->getType() == Annot::typeScreen) {
 	AnnotScreen *annot = static_cast<AnnotScreen *>(annots->getAnnot(i));
 	scanLinkAction(annot->getAction(),
-                       "Screen Annotation Activated", false);
-	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorEntering),
+                       "Screen Annotation Activated");
+	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorEntering).get(),
                        "Screen Annotation Cursor Enter");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorLeaving),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorLeaving).get(),
                        "Screen Annotation Cursor Leave");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionMousePressed),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionMousePressed).get(),
                        "Screen Annotation Mouse Pressed");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionMouseReleased),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionMouseReleased).get(),
                        "Screen Annotation Mouse Released");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusIn),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusIn).get(),
                        "Screen Annotation Focus In");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusOut),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusOut).get(),
                        "Screen Annotation Focus Out");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageOpening),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageOpening).get(),
                        "Screen Annotation Page Open");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageClosing),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageClosing).get(),
                        "Screen Annotation Page Close");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageVisible),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageVisible).get(),
                        "Screen Annotation Page Visible");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageInvisible),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageInvisible).get(),
                        "Screen Annotation Page Invisible");
 
       } else if (annots->getAnnot(i)->getType() == Annot::typeWidget) {
 	AnnotWidget *annot = static_cast<AnnotWidget *>(annots->getAnnot(i));
 	scanLinkAction(annot->getAction(),
-                       "Widget Annotation Activated", false);
-	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorEntering),
+                       "Widget Annotation Activated");
+	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorEntering).get(),
                        "Widget Annotation Cursor Enter");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorLeaving),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionCursorLeaving).get(),
                        "Widget Annotation Cursor Leave");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionMousePressed),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionMousePressed).get(),
                        "Widget Annotation Mouse Pressed");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionMouseReleased),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionMouseReleased).get(),
                        "Widget Annotation Mouse Released");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusIn),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusIn).get(),
                        "Widget Annotation Focus In");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusOut),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionFocusOut).get(),
                        "Widget Annotation Focus Out");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageOpening),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageOpening).get(),
                        "Widget Annotation Page Open");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageClosing),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageClosing).get(),
                        "Widget Annotation Page Close");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageVisible),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageVisible).get(),
                        "Widget Annotation Page Visible");
-	scanLinkAction(annot->getAdditionalAction(Annot::actionPageInvisible),
+	scanLinkAction(annot->getAdditionalAction(Annot::actionPageInvisible).get(),
                        "Widget Annotation Page Invisible");
       }
     }
